package com.spcapitaliq.sbtadhocsubprojects

import java.io.File
import sbt._

object SbtAdhocSubprojectsPlugin extends Plugin {
  import Keys._

  object Keys {
    val checkoutDependencies: SettingKey[Checkouts] =
      SettingKey("checkout-dependencies", "Ad-hoc source dependencies in use.")
  }

  type Checkouts = Seq[String]

  private type CheckoutDependencies = Seq[ClasspathDep[ProjectRef]]

  /** Subdirectory containing checkouts. */
  def checkouts: String = "checkouts"

  /** List basenames of directories in `dir`. */
  private def findCheckouts(dir: String): Checkouts =
    (Option(new File(dir) list).toSeq.flatten.toIndexedSeq.view
     filterNot Set(".", "..")
     filter (new File(dir, _).isDirectory) force)

  /** Discover adhoc subprojects. */
  private def checkoutProjects(checkoutDir: String): Seq[Project] =
    findCheckouts(checkoutDir) map (sd =>
      Project("checkouts--" + sd, new File(checkoutDir, sd)))

  /** Main entry point; discover and install adhoc subprojects. */
  def withAdhocSubprojects(pjs: Seq[Project], checkoutDir: String = checkouts
                         ): Seq[Project] = {
    val basenames = findCheckouts(checkoutDir)
    val adhoc = checkoutProjects(checkoutDir)
    val adhocDeps = adhoc map (ClasspathDependency(_, None))
    pjs.map {p =>
      // the adhocDeps need to go first
      p.copy(dependencies = adhocDeps ++ p.dependencies)
       .settings(checkoutDependencies := basenames)
    } ++ adhoc
  }
}
